/** Fr den Keylistener und so brauchen wir einige AWT-Pakete.*/
import java.awt.event.*;
import java.awt.*;
/**
 * Die Klasse BoulderDashGUI generiert die GUI fr Boulder Dash.
 * 
 * @author (Marc A. Modrow) 
 * @version (9.02.08)
 */
public class BoulderDash extends Frame implements KeyListener, WindowListener, ActionListener {

    class Wrapper extends BoulderDashBackEnd{
        private int imageSize = 32;
        private Image[] images = new Image[96];
        private Font font = new Font("SansSerif", Font.BOLD, imageSize);
        
        public Wrapper(){
            images[' '] = Toolkit.getDefaultToolkit().getImage("empty.png");
            images['M'] = Toolkit.getDefaultToolkit().getImage("wall.png");
            images['E'] = Toolkit.getDefaultToolkit().getImage("earth.png");
            images['A'] = Toolkit.getDefaultToolkit().getImage("exit.png");
            images['S'] = Toolkit.getDefaultToolkit().getImage("player.png");
            images['O'] = Toolkit.getDefaultToolkit().getImage("stone.png");
            images['U'] = Toolkit.getDefaultToolkit().getImage("monster.png");            
            images['!'] = Toolkit.getDefaultToolkit().getImage("treasure.png"); 
            images['X'] = Toolkit.getDefaultToolkit().getImage("killed.png"); 
            gameArea.setPreferredSize(new Dimension(field[0].length()*imageSize, (field.length + 3) * imageSize));
        }
        
       
        protected void print(){
            repaint();
            super.print();
        }
        

        void paint(Graphics g){
            for (int y = 0; y < field.length; ++y){
                for (int x = 0; x < field[0].length(); ++x){
                    if(fieldChar[y][x] == 'U'){
                        switch(fieldMob[y][x].direction){
                            case 0: g.drawImage(Toolkit.getDefaultToolkit().getImage("monstero.png"), x * imageSize, y * imageSize, imageSize, imageSize, BoulderDash.this);
                            break;
                            case 1: g.drawImage(Toolkit.getDefaultToolkit().getImage("monsterl.png"), x * imageSize, y * imageSize, imageSize, imageSize, BoulderDash.this);
                            break;
                            case 2: g.drawImage(Toolkit.getDefaultToolkit().getImage("monsteru.png"), x * imageSize, y * imageSize, imageSize, imageSize, BoulderDash.this);
                            break;
                            case 3: g.drawImage(Toolkit.getDefaultToolkit().getImage("monsterr.png"), x * imageSize, y * imageSize, imageSize, imageSize, BoulderDash.this);
                            break;
                        }
                    }else if(fieldChar[y][x] == 'A'){
                        if(game.treasures==0){
                            g.drawImage(Toolkit.getDefaultToolkit().getImage("exitopen.png"), x * imageSize, y * imageSize, imageSize, imageSize, BoulderDash.this);
                        }else{
                            g.drawImage(Toolkit.getDefaultToolkit().getImage("exitclosed.png"), x * imageSize, y * imageSize, imageSize, imageSize, BoulderDash.this);
                        }
                    }
                    else{
                    g.drawImage(images[fieldChar[y][x]], x * imageSize, y * imageSize, imageSize, imageSize, BoulderDash.this);
                    }
                }
            }
            g.setFont(font);
            int width = g.getFontMetrics().stringWidth(message);
            int x = (field[0].length()*imageSize-width)/2;
            int y = (field.length+1)*imageSize;
            g.setColor(Color.BLACK);
            g.drawString(message, x,y + imageSize - g.getFontMetrics().getMaxDescent());
        }
    }
        
        
    Wrapper game;   
    Canvas gameArea;

    BoulderDash(){
        super("Boulder Dash");
        gameArea = new Canvas(){
            public void paint(Graphics g){
                game.paint(g);
            }
        };
        add(gameArea);
        gameArea.addKeyListener(this);
        game = new Wrapper();
        addWindowListener(this);
        setMenuBar(createMenu());
        pack();
        setVisible(true);
    }
    
    public void paint(Graphics g){
        gameArea.repaint();
    }
    
    
    
    public void actionPerformed(ActionEvent e){
        if(e.getActionCommand().equals("Neues Spiel starten")){
            game.reset();
        } else if(e.getActionCommand().equals("Beenden")){
            windowClosing(null);
        } else if(e.getActionCommand().equals("ber Boulder Dash...")){
            new AboutBox(this);
        }
    }
    private MenuBar createMenu(){
        MenuBar mb = new MenuBar();
        Menu m = new Menu("Spiel");
        MenuItem mi = new MenuItem("Neues Spiel starten");
        mi.addActionListener(this);
        m.add(mi);
        m.addSeparator();
        mi = new MenuItem("Beenden");
        mi.addActionListener(this);
        m.add(mi);
        mb.add(m);
        m = new Menu("Hilfe");
        mi = new MenuItem("ber Boulder Dash...");
        mi.addActionListener(this);
        m.add(mi);
        mb.setHelpMenu(m);
        return mb;
    }
    
    public void windowClosing(WindowEvent e){
        dispose();
        System.exit(0);
    }
    public void windowClosed(WindowEvent e){}
    public void windowOpening(WindowEvent e){}
    public void windowOpened(WindowEvent e){}
    public void windowIconified(WindowEvent e){}
    public void windowDeiconified(WindowEvent e){}        
    public void windowActivated(WindowEvent e){}
    public void windowDeactivated(WindowEvent e){}
        
    public void keyPressed(KeyEvent e) {
        try{
            if (game.playerExists == true){
                switch (e.getKeyCode()) {
                    case KeyEvent.VK_UP:
                    case KeyEvent.VK_W:
                        game.step(0);
                        break;
                    case KeyEvent.VK_LEFT:
                    case KeyEvent.VK_A:
                        game.step(1);
                        break;
                    case KeyEvent.VK_DOWN:
                    case KeyEvent.VK_S:
                        game.step(2);
                        break;
                    case KeyEvent.VK_RIGHT:
                    case KeyEvent.VK_D:
                        game.step(3);
                        break;
                    case KeyEvent.VK_X:
                        game.step(4);
                        break;
                }
                game.print();
            }
        }
        catch (Exception f) {
            game.message = f.getMessage();
            game.playerExists = false;
            repaint();
        }
    }

    /** Die Methode keyReleased ist ntig fr den Keylistener, tut aber nichts.*/
    public void keyReleased(KeyEvent e) {
        if(e.getKeyCode() == KeyEvent.VK_F1){
            game.reset();
        }
        else if(e.getKeyCode() == KeyEvent.VK_ESCAPE){
            dispose();
        }
    }
    /** Die Methode keyTyped ist ntig fr den Keylistener, tut aber nichts.*/
    public void keyTyped(KeyEvent e) {
    }
    
    /** Die Methode main interpretiert gegen Eingabe eines String-Arrays die Strings als
     *  w-a-s-d-Steuerung + x zum Warten und ruft game mit den dazugehrigen Werten 0-4
     *  fr die einzelnen Spielzge auf.
     *  @param args ist die Eingabe des Benutzers fr die wasdx-Steuerung.
     */
    public static void main(String[] args){

        if(args.length == 0){
        new BoulderDash();
        }
    }
}
